#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_months
----------------------------------

Tests for `months` module.
"""

import os
import sys
import datetime
import unittest

sys.path.append(os.path.join('.', 'months'))
sys.path.append(os.path.join('..', 'months'))

from months import Month


class TestMonths(unittest.TestCase):

    def setUp(self):
        self.datetime = datetime.datetime(2015, 4, 15)
        self.date = self.datetime.date()
        self.month = Month(2015, 4)

    def test_repr(self):
        self.assertEqual(repr(self.month), 'Month(2015, 4)')

    def test_str(self):
        self.assertEqual(str(self.month), '2015-04')

    def test_month_name(self):
        self.assertEqual(self.month.month_name, 'April')

    def test_month_abbr(self):
        self.assertEqual(self.month.month_abbr, 'Apr')

    def test_full_display(self):
        self.assertEqual(self.month.full_display, 'April 2015')

    def test_abbr_display(self):
        self.assertEqual(self.month.abbr_display, 'Apr 2015')

    def test_from_datetime(self):
        self.assertEqual(self.month, Month.from_date(self.datetime))

    def test_from_date(self):
        self.assertEqual(self.month, Month.from_date(self.date))

    def test_add(self):
        self.assertEqual(self.month + 1, Month(2015, 5))

    def test_add_rollover(self):
        self.assertEqual(self.month + 9, Month(2016, 1))

    def test_add_raises(self):
        self.assertRaises(ValueError, self.month.__add__, 'not_an_int')

    def test_sub(self):
        self.assertEqual(self.month - 1, Month(2015, 3))

    def test_sub_rollover(self):
        self.assertEqual(self.month - 4, Month(2014, 12))

    def test_sub_raises(self):
        self.assertRaises(ValueError, self.month.__sub__, 'not_an_int')

    def test_start_date(self):
        self.assertEqual(self.month.start_date, self.date.replace(day=1))

    def test_end_date(self):
        self.assertEqual(self.month.end_date, self.date.replace(day=30))

    def test_range(self):
        self.assertEqual(
            self.month.range,
            (self.date.replace(day=1), self.date.replace(day=30)))

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from collections import namedtuple
import calendar
import datetime


def __utctoday():
    """ Returns today's date in UTC time """
    return datetime.datetime.utcnow().date()


class Month(namedtuple('Month', ['year', 'month'])):
    """
    Represents a specific month of a year

    Provides various utilites for generating, manipulating, and displaying
    months.
    """
    def __repr__(self):
        return "%s(%d, %d)" % (self.__class__.__name__, self.year, self.month)

    def __str__(self):
        return self.start_date.strftime("%Y-%m")

    @property
    def month_name(self):
        """
        The calendar name of the month.

        >>> Month(2015, 4).month_name
        'April'
        """
        return calendar.month_name[self.month]

    @property
    def month_abbr(self):
        """
        The abbreviated calendar name of the month.

        >>> Month(2015, 4).month_abbr
        'Apr'
        """
        return calendar.month_abbr[self.month]

    @property
    def full_display(self):
        """
        The calendar name of the month along with the year

        >>> Month(2015, 4).full_display
        'April 2015'
        """
        return "%s %d" % (self.month_name, self.year)

    @property
    def abbr_display(self):
        """
        The abbreviated calendar name of the month along with the year

        >>> Month(2015, 4).full_display
        'Apr 2015'
        """
        return "%s %d" % (self.month_abbr, self.year)

    @classmethod
    def from_date(cls, date):
        """
        Returns a Month instance from the given datetime.date or
        datetime.datetime object
        """
        try:
            date = date.date()
        except AttributeError:
            pass
        return cls(date.year, date.month)

    @classmethod
    def from_today(cls):
        """
        Returns a Month instance from today's date (local time)
        """
        return cls.from_date(datetime.date.today())

    @classmethod
    def from_utc_today(cls):
        """
        Returns a Month instance from today's date (UTC time)
        """
        return cls.from_date(__utctoday())

    def __add__(self, other):
        """
        Offsets a number of months into the future

        >>> Month(2015, 4) + 9
        Month(2016, 1)
        """
        if not isinstance(other, int):
            raise ValueError("Only ints can be added to months")

        year_change, month = divmod(self.month + other - 1, 12)
        return type(self)(self.year + year_change, month + 1)

    def __sub__(self, other):
        """
        Offsets a number of months into the past

        >>> Month(2015, 4) - 9
        Month(2014, 7)
        """
        if not isinstance(other, int):
            raise ValueError("Only ints can be added to months")

        return self + (-other)

    @property
    def start_date(self):
        """ Returns a datetime.date object for the first day of the month """
        return datetime.date(self.year, self.month, 1)

    @property
    def end_date(self):
        """ Returns a datetime.date object for the last day of the month """
        return (self + 1).start_date - datetime.timedelta(1)

    @property
    def range(self):
        """ Returns a tuple of the first and last days of the month """
        return (self.start_date, self.end_date)
